/****************************************************************************
** Meta object code from reading C++ file 'ThumbsViewer.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "ThumbsViewer.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ThumbsViewer.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12ThumbsViewerE_t {};
} // unnamed namespace

template <> constexpr inline auto ThumbsViewer::qt_create_metaobjectdata<qt_meta_tag_ZN12ThumbsViewerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ThumbsViewer",
        "currentIndexChanged",
        "",
        "QModelIndex",
        "current",
        "filesHidden",
        "files",
        "filesShown",
        "progress",
        "total",
        "status",
        "s",
        "lazySelectionChanged",
        "count",
        "invertSelection",
        "filterRows",
        "first",
        "last",
        "loadVisibleThumbs",
        "scrollBarValue",
        "guarded",
        "promoteSelectionChange",
        "tagSelected",
        "tagsAdded",
        "tagsRemoved",
        "updateThumbnail",
        "fileName",
        "currentChanged",
        "previous",
        "loadThumbsRange"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'currentIndexChanged'
        QtMocHelpers::SignalData<void(const QModelIndex &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'filesHidden'
        QtMocHelpers::SignalData<void(const QStringList &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 6 },
        }}),
        // Signal 'filesShown'
        QtMocHelpers::SignalData<void(const QStringList &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 6 },
        }}),
        // Signal 'progress'
        QtMocHelpers::SignalData<void(unsigned int, unsigned int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 4 }, { QMetaType::UInt, 9 },
        }}),
        // Signal 'status'
        QtMocHelpers::SignalData<void(QString)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 11 },
        }}),
        // Signal 'lazySelectionChanged'
        QtMocHelpers::SignalData<void(int)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 13 },
        }}),
        // Slot 'invertSelection'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'filterRows'
        QtMocHelpers::SlotData<void(int, int)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 16 }, { QMetaType::Int, 17 },
        }}),
        // Slot 'filterRows'
        QtMocHelpers::SlotData<void(int)>(15, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 16 },
        }}),
        // Slot 'filterRows'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'loadVisibleThumbs'
        QtMocHelpers::SlotData<void(int, bool)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 19 }, { QMetaType::Bool, 20 },
        }}),
        // Slot 'loadVisibleThumbs'
        QtMocHelpers::SlotData<void(int)>(18, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 19 },
        }}),
        // Slot 'loadVisibleThumbs'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'promoteSelectionChange'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'tagSelected'
        QtMocHelpers::SlotData<void(const QStringList &, const QStringList &) const>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 23 }, { QMetaType::QStringList, 24 },
        }}),
        // Slot 'updateThumbnail'
        QtMocHelpers::SlotData<void(const QString &)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 26 },
        }}),
        // Slot 'currentChanged'
        QtMocHelpers::SlotData<void(const QModelIndex &, const QModelIndex &)>(27, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 3, 28 },
        }}),
        // Slot 'loadThumbsRange'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ThumbsViewer, qt_meta_tag_ZN12ThumbsViewerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ThumbsViewer::staticMetaObject = { {
    QMetaObject::SuperData::link<QListView::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12ThumbsViewerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12ThumbsViewerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12ThumbsViewerE_t>.metaTypes,
    nullptr
} };

void ThumbsViewer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ThumbsViewer *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->currentIndexChanged((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 1: _t->filesHidden((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 2: _t->filesShown((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 3: _t->progress((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[2]))); break;
        case 4: _t->status((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->lazySelectionChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->invertSelection(); break;
        case 7: _t->filterRows((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 8: _t->filterRows((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 9: _t->filterRows(); break;
        case 10: _t->loadVisibleThumbs((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 11: _t->loadVisibleThumbs((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 12: _t->loadVisibleThumbs(); break;
        case 13: _t->promoteSelectionChange(); break;
        case 14: _t->tagSelected((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2]))); break;
        case 15: _t->updateThumbnail((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 16: _t->currentChanged((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2]))); break;
        case 17: _t->loadThumbsRange(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ThumbsViewer::*)(const QModelIndex & )>(_a, &ThumbsViewer::currentIndexChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ThumbsViewer::*)(const QStringList & )>(_a, &ThumbsViewer::filesHidden, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ThumbsViewer::*)(const QStringList & )>(_a, &ThumbsViewer::filesShown, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ThumbsViewer::*)(unsigned int , unsigned int )>(_a, &ThumbsViewer::progress, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ThumbsViewer::*)(QString )>(_a, &ThumbsViewer::status, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ThumbsViewer::*)(int )>(_a, &ThumbsViewer::lazySelectionChanged, 5))
            return;
    }
}

const QMetaObject *ThumbsViewer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ThumbsViewer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12ThumbsViewerE_t>.strings))
        return static_cast<void*>(this);
    return QListView::qt_metacast(_clname);
}

int ThumbsViewer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QListView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 18;
    }
    return _id;
}

// SIGNAL 0
void ThumbsViewer::currentIndexChanged(const QModelIndex & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void ThumbsViewer::filesHidden(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void ThumbsViewer::filesShown(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void ThumbsViewer::progress(unsigned int _t1, unsigned int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void ThumbsViewer::status(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void ThumbsViewer::lazySelectionChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}
QT_WARNING_POP
