/****************************************************************************
** Meta object code from reading C++ file 'Phototonic.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "Phototonic.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'Phototonic.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10PhototonicE_t {};
} // unnamed namespace

template <> constexpr inline auto Phototonic::qt_create_metaobjectdata<qt_meta_tag_ZN10PhototonicE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Phototonic",
        "setSaveDirectory",
        "",
        "path",
        "loadSelectedThumbImage",
        "QModelIndex",
        "idx",
        "showViewer",
        "hideViewer",
        "dropOp",
        "Qt::KeyboardModifiers",
        "keyMods",
        "dirOp",
        "copyMoveDirPath",
        "sortThumbnails",
        "setFileListMode",
        "on",
        "reload",
        "setIncludeSubDirs",
        "showSettings",
        "toggleFullScreen",
        "updateActions",
        "reloadThumbs",
        "renameDir",
        "setImageViewerWindowTitle",
        "setThumbsViewerWindowTitle",
        "rename",
        "removeMetadata",
        "viewImage",
        "newImage",
        "deleteDirectory",
        "trash",
        "createSubDirectory",
        "checkDirState",
        "goPathBarDir",
        "goTo",
        "setSlideShow",
        "active",
        "slideShowHandler",
        "selectAllThumbs",
        "deleteOperation",
        "deletePermanentlyOperation",
        "pasteThumbs",
        "destDir",
        "resizeThumbs",
        "rotate",
        "deg",
        "flipVertical",
        "cropImage",
        "scaleImage",
        "freeRotate",
        "batchTransform",
        "showColorsDialog",
        "flipHorizontal",
        "setDocksVisibility",
        "visible",
        "showHiddenFiles",
        "setToolbarIconSize",
        "chooseExternalApp",
        "updateExternalApps",
        "runExternalApp",
        "lockDocks",
        "filterImagesFocus",
        "setPathFocus",
        "regenerateThumbnails"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setSaveDirectory'
        QtMocHelpers::SlotData<void(QString)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'setSaveDirectory'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'loadSelectedThumbImage'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(4, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'showViewer'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'hideViewer'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'dropOp'
        QtMocHelpers::SlotData<void(Qt::KeyboardModifiers, bool, QString)>(9, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 10, 11 }, { QMetaType::Bool, 12 }, { QMetaType::QString, 13 },
        }}),
        // Slot 'sortThumbnails'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setFileListMode'
        QtMocHelpers::SlotData<void(bool)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 16 },
        }}),
        // Slot 'reload'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setIncludeSubDirs'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showSettings'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'toggleFullScreen'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateActions'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'reloadThumbs'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'renameDir'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setImageViewerWindowTitle'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setThumbsViewerWindowTitle'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'rename'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'removeMetadata'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'viewImage'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'newImage'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'deleteDirectory'
        QtMocHelpers::SlotData<void(bool)>(30, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 31 },
        }}),
        // Slot 'createSubDirectory'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'checkDirState'
        QtMocHelpers::SlotData<void(const QModelIndex &, int, int)>(33, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 5, 2 }, { QMetaType::Int, 2 }, { QMetaType::Int, 2 },
        }}),
        // Slot 'goPathBarDir'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'goTo'
        QtMocHelpers::SlotData<void(QString)>(35, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'setSlideShow'
        QtMocHelpers::SlotData<void(bool)>(36, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 37 },
        }}),
        // Slot 'slideShowHandler'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'selectAllThumbs'
        QtMocHelpers::SlotData<void()>(39, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'deleteOperation'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'deletePermanentlyOperation'
        QtMocHelpers::SlotData<void()>(41, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'pasteThumbs'
        QtMocHelpers::SlotData<void(QString)>(42, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 43 },
        }}),
        // Slot 'pasteThumbs'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void),
        // Slot 'resizeThumbs'
        QtMocHelpers::SlotData<void()>(44, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'rotate'
        QtMocHelpers::SlotData<void(int)>(45, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 46 },
        }}),
        // Slot 'flipVertical'
        QtMocHelpers::SlotData<void()>(47, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'cropImage'
        QtMocHelpers::SlotData<void()>(48, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'scaleImage'
        QtMocHelpers::SlotData<void()>(49, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'freeRotate'
        QtMocHelpers::SlotData<void(int)>(50, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 46 },
        }}),
        // Slot 'batchTransform'
        QtMocHelpers::SlotData<void()>(51, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showColorsDialog'
        QtMocHelpers::SlotData<void()>(52, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'flipHorizontal'
        QtMocHelpers::SlotData<void()>(53, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setDocksVisibility'
        QtMocHelpers::SlotData<void(bool)>(54, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 55 },
        }}),
        // Slot 'showHiddenFiles'
        QtMocHelpers::SlotData<void()>(56, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setToolbarIconSize'
        QtMocHelpers::SlotData<void()>(57, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'chooseExternalApp'
        QtMocHelpers::SlotData<void()>(58, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateExternalApps'
        QtMocHelpers::SlotData<void()>(59, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'runExternalApp'
        QtMocHelpers::SlotData<void()>(60, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'lockDocks'
        QtMocHelpers::SlotData<void()>(61, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'filterImagesFocus'
        QtMocHelpers::SlotData<void()>(62, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setPathFocus'
        QtMocHelpers::SlotData<void()>(63, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'regenerateThumbnails'
        QtMocHelpers::SlotData<void()>(64, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Phototonic, qt_meta_tag_ZN10PhototonicE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Phototonic::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10PhototonicE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10PhototonicE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10PhototonicE_t>.metaTypes,
    nullptr
} };

void Phototonic::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Phototonic *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setSaveDirectory((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->setSaveDirectory(); break;
        case 2: _t->loadSelectedThumbImage((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 3: _t->showViewer(); break;
        case 4: _t->hideViewer(); break;
        case 5: _t->dropOp((*reinterpret_cast<std::add_pointer_t<Qt::KeyboardModifiers>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 6: _t->sortThumbnails(); break;
        case 7: _t->setFileListMode((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->reload(); break;
        case 9: _t->setIncludeSubDirs(); break;
        case 10: _t->showSettings(); break;
        case 11: _t->toggleFullScreen(); break;
        case 12: _t->updateActions(); break;
        case 13: _t->reloadThumbs(); break;
        case 14: _t->renameDir(); break;
        case 15: _t->setImageViewerWindowTitle(); break;
        case 16: _t->setThumbsViewerWindowTitle(); break;
        case 17: _t->rename(); break;
        case 18: _t->removeMetadata(); break;
        case 19: _t->viewImage(); break;
        case 20: _t->newImage(); break;
        case 21: _t->deleteDirectory((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 22: _t->createSubDirectory(); break;
        case 23: _t->checkDirState((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 24: _t->goPathBarDir(); break;
        case 25: _t->goTo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 26: _t->setSlideShow((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 27: _t->slideShowHandler(); break;
        case 28: _t->selectAllThumbs(); break;
        case 29: _t->deleteOperation(); break;
        case 30: _t->deletePermanentlyOperation(); break;
        case 31: _t->pasteThumbs((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 32: _t->pasteThumbs(); break;
        case 33: _t->resizeThumbs(); break;
        case 34: _t->rotate((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 35: _t->flipVertical(); break;
        case 36: _t->cropImage(); break;
        case 37: _t->scaleImage(); break;
        case 38: _t->freeRotate((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 39: _t->batchTransform(); break;
        case 40: _t->showColorsDialog(); break;
        case 41: _t->flipHorizontal(); break;
        case 42: _t->setDocksVisibility((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 43: _t->showHiddenFiles(); break;
        case 44: _t->setToolbarIconSize(); break;
        case 45: _t->chooseExternalApp(); break;
        case 46: _t->updateExternalApps(); break;
        case 47: _t->runExternalApp(); break;
        case 48: _t->lockDocks(); break;
        case 49: _t->filterImagesFocus(); break;
        case 50: _t->setPathFocus(); break;
        case 51: _t->regenerateThumbnails(); break;
        default: ;
        }
    }
}

const QMetaObject *Phototonic::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Phototonic::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10PhototonicE_t>.strings))
        return static_cast<void*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int Phototonic::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 52)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 52;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 52)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 52;
    }
    return _id;
}
QT_WARNING_POP
